/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ProgressBuilder;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;

// ToolBox imports
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.io.PrintWriter;
import java.io.File;
import java.util.StringTokenizer;

//
abstract public class AbstractEchoBuilder extends ProgressBuilder {
    private boolean      echo = true;
    private OtterProject project = null;
    private PrintWriter  echoWriter = null;
    private Writer       capture = null;

    /**
     * Create a CompileThread with listeners ready to
     * recieve output from build.
     *
     *
     * @param listeners
     * A listener implements an onWrite method that is called
     * to write output. The writes are captured for display or
     * logging.
     */
    public AbstractEchoBuilder() {
        super();
        capture = new Writer();
        capture.setLineBuffered(false);
        echoWriter = new PrintWriter(capture, false);
    }

    public AbstractEchoBuilder(WriteListener listener) {
        this(listener, false);
    }

    public AbstractEchoBuilder(WriteListener listener, boolean buffer) {
        super();
        capture = new Writer();
        capture.setLineBuffered(buffer);
        addWriteListener(listener);
        echoWriter = new PrintWriter(capture, false);
    }

    /**
     * Get the IDE project that contains the HTML source files to compile.
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Set the IDE project that contains the HTML source files to compile.
     */
    public void setProject(OtterProject bp) {
        project = bp;
    }

    public boolean isEcho() {
        return echo;
    }

    public void setEcho(boolean e) {
        echo = e;
    }

    public PrintWriter getEchoWriter() {
        return echoWriter;
    }

    public void echo(String message) {
        if (isEcho()) {
            getEchoWriter().println(message);
        }
    }

    public void echo(Exception e) {
        StringTokenizer tokenizer = null;
        String          message = null;

        if (e != null) {
            message = e.getMessage();
            if (message == null) {
              message = e.toString();
            }
            e.printStackTrace(System.err);
        }
        if (message != null) {
            tokenizer = new StringTokenizer(message, Constants.NEWLINE);
            while (tokenizer.hasMoreTokens()) {
                getEchoWriter().println(tokenizer.nextToken());
            }
        }
    }

    /**
     * Run the XMLC for all selected HTML files.
     */
    public synchronized void build() {
        setFresh(true);
        BuildRunner runner = new BuildRunner();

        runner.start();
    }

    public synchronized void buildInCurrentThread() {
        setFresh(true);
        try {
            buildImpl();
        } catch (Exception e) {
            echo(e);
        }
    }

    protected WriteListener getWriteListener() {
        WriteListener listener = null;

        if (capture.getWriteListeners().length > 0) {
            listener = capture.getWriteListeners()[0];
        }
        return listener;
    }

    protected void addWriteListener(WriteListener w) {
        if (w != null) {
            capture.addWriteListener(w);
        }
    }

    protected void removeWriteListener(WriteListener w) {
        if (w != null) {
            capture.removeWriteListener(w);
        }
    }

    /**
     * Loop through all selected nodes files and build them.
     * Write out exceptions to any registered listeners.
     */
    abstract protected void buildImpl() throws ToolException;

    /**
     * Sleep the thread for a given number of seconds.
     *
     *
     * @param seconds
     * How many seconds to sleep.
     */
    public void sleepInSeconds(double seconds) {
        try {
            Thread.sleep((int) (seconds * 1000));
        } catch (InterruptedException e) {}
    }

    private class BuildRunner extends Thread {
        protected BuildRunner() {
            setPriority((int) ((Thread.MIN_PRIORITY + Thread.NORM_PRIORITY)
                               / 2));
        }

        public void run() {
            try {
                buildImpl();
            } catch (Exception e) {
                echo(e);
            }
        }

    }
}
