/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.util.Arrays;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.beans.*;

//
public class ArchivePanel extends JPanel implements Instructor, DeployStep {
    private GridBagLayout       layoutMain;
    private JScrollPane         scroll;
    private ArchiveTablePanel   archiveTable;
    private JPanel              panelButton;
    private JButton             buttonAdd;
    private JButton             buttonRemove;
    private GridBagLayout       layoutButton;
    private JButton             buttonUp;
    private JButton             buttonDown;
    private LocalButtonListener buttonListener = null;
    private boolean             ran = false;
    private JButton             buttonEdit;

    public ArchivePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // implements Instructor
    public String getTab() {
        return "Archive";
    }

    // implements Instructor
    public String getTitle() {
        return "Deployable archives";
    }

    // implements Instructor
    public String getInstructions() {
        return "The deployer can recreate archives that you can auto-deploy "
               + "to a running Enhydra application server.";
    }

    // implements DeployStep
    public void refresh() {

        // nothing to refresh
    }

    // implements DeployStep
    public void clearAll() {
        archiveTable.clearAll();
        buttonAdd.removeActionListener(buttonListener);
        buttonRemove.removeActionListener(buttonListener);
        buttonUp.removeActionListener(buttonListener);
        buttonDown.removeActionListener(buttonListener);
        removeAll();
        buttonListener = null;
    }

    // implements DeployStep
    public boolean isDataValid() {
        return DeployStepUtil.isDataValid(this);
    }

    // implements DeployStep
    public boolean isDataEqual(OtterProject project) {
        boolean   equal = true;
        JarPlan[] projectPlans = new JarPlan[0];
        JarPlan[] panelPlans = new JarPlan[0];

        projectPlans = project.getArchivePlans(false);
        panelPlans = archiveTable.getPlans();
        if (!Arrays.equals(panelPlans, projectPlans)) {
            equal = false;
        }
        return equal;
    }

    // implements DeployStep
    public void validateData() throws DataValidationException {
        JarPlan[]  plans = new JarPlan[0];
        PathHandle path = null;

        plans = archiveTable.getPlans();
        for (int i = 0; i < plans.length; i++) {
            path = PathHandle.createPathHandle(plans[i].getArchivePath());
            if (path.isDirectory()) {
                throw new DataValidationException("Invalid archive: "
                                                  + path.getPath());
            }
        }
    }

    // implements DeployStep
    public boolean isBuilt(OtterProject project) {
        ran = DeployStepUtil.isBuilt(this, ran, project);
        return ran;
    }

    // implements DeployStep
    public boolean isSelectable(OtterProject project, DeployStep[] steps) {
        boolean able = false;
        boolean generalBuilt = false;
        boolean inputBuilt = false;
        boolean contentBuilt = false;

        for (int i = 0; i < steps.length; i++) {
            if (steps[i] instanceof GeneralPanel) {
                generalBuilt = steps[i].isBuilt(project);
            } else if (steps[i] instanceof InputPanel) {
                inputBuilt = steps[i].isBuilt(project);
            } else if (steps[i] instanceof ContentPanel) {
                contentBuilt = steps[i].isBuilt(project);
            }
        }
        able = generalBuilt && inputBuilt && contentBuilt;
        return able;
    }

    // implements DeployStep
    public void build(OtterProject project,
                      WriteListener listener) throws DataValidationException {
        ArchiveBuilder builder = null;

        if (!isBuilt(project)) {
            write(project);
            builder = new ArchiveBuilder(listener);
            builder.setEcho(true);
            builder.setProject(project);
            builder.buildInCurrentThread();
            ran = true;
        }
    }

    // implements DeployStep
    public void read(OtterProject project) {
        PathHandle ph = null;
        JarPlan[]  plans = new JarPlan[0];

        // TODO: Pass project root to reader for relitive path expansion.
        plans = project.getArchivePlans(false);
        archiveTable.setPlans(plans);
        ph =
            PathHandle.createPathHandle(PathUtil.getDefaultDeployArchivePath(project));
        archiveTable.setDefWorkingRoot(ph.getParent().getPath());
    }

    // implements DeployStep
    public void write(OtterProject project) throws DataValidationException {
        JarPlan[] plans = new JarPlan[0];

        validateData();
        if (!isDataEqual(project)) {
            plans = archiveTable.getPlans();
            project.setArchivePlans(plans);
        }
    }

    //
    //
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonAdd.addActionListener(buttonListener);
        buttonEdit.addActionListener(buttonListener);
        buttonRemove.addActionListener(buttonListener);
        buttonUp.addActionListener(buttonListener);
        buttonDown.addActionListener(buttonListener);
    }

    private void jbInit() throws Exception {
        scroll = (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                                 JScrollPane.class.getName());
        archiveTable =
            (ArchiveTablePanel) Beans.instantiate(getClass().getClassLoader(),
                                                  ArchiveTablePanel.class.getName());
        panelButton = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutButton =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonUp = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                               JButton.class.getName());
        buttonDown = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonEdit = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonAdd.setMaximumSize(new Dimension(100, 27));
        buttonAdd.setMinimumSize(new Dimension(100, 27));
        buttonAdd.setPreferredSize(new Dimension(100, 27));
        buttonAdd.setText("Add...");
        buttonRemove.setMaximumSize(new Dimension(100, 27));
        buttonRemove.setMinimumSize(new Dimension(100, 27));
        buttonRemove.setPreferredSize(new Dimension(100, 27));
        buttonRemove.setText("Remove");
        panelButton.setLayout(layoutButton);
        buttonUp.setMaximumSize(new Dimension(100, 27));
        buttonUp.setMinimumSize(new Dimension(100, 27));
        buttonUp.setPreferredSize(new Dimension(100, 27));
        buttonUp.setText("Move Up");
        buttonDown.setMaximumSize(new Dimension(100, 27));
        buttonDown.setMinimumSize(new Dimension(100, 27));
        buttonDown.setPreferredSize(new Dimension(100, 27));
        buttonDown.setText("Move Down");
        buttonEdit.setMaximumSize(new Dimension(100, 27));
        buttonEdit.setMinimumSize(new Dimension(100, 27));
        buttonEdit.setPreferredSize(new Dimension(100, 27));
        buttonEdit.setText("Edit...");
        panelButton.add(buttonAdd,
                        new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(20, 5, 5, 5), 0,
                                               0));
        panelButton.add(buttonRemove,
                        new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(buttonUp,
                        new GridBagConstraints(0, 3, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(buttonDown,
                        new GridBagConstraints(0, 4, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 20, 5), 0,
                                               0));
        panelButton.add(buttonEdit,
                        new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        scroll.getViewport().add(archiveTable, null);
        this.setLayout(layoutMain);
        this.add(scroll,
                 new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(panelButton,
                 new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            try {
                if (source == buttonAdd) {
                    archiveTable.addRow();
                } else if (source == buttonEdit) {
                    archiveTable.editRow();
                } else if (source == buttonRemove) {
                    archiveTable.removeSelection();
                } else if (source == buttonDown) {
                    archiveTable.moveSelectionDown();
                } else if (source == buttonUp) {
                    archiveTable.moveSelectionUp();
                }
            } catch (ArchiveException e) {
                JOptionPane.showMessageDialog(getTopLevelAncestor(),
                                              e.getMessage());
            }
        }

    }
}
