/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// Kelp imports
import org.enhydra.kelp.common.AbstractEchoBuilder;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Template;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.configure.ConfigTool;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

//
public class InputBuilder extends AbstractEchoBuilder {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private File[]              builtTemplates = new File[0];
    private File[]              builtPassthrough = new File[0];
    private OtterNode[] nodes = null;
    private Component           owner = null;

    public InputBuilder(WriteListener listener) {
        super(listener);
    }

    public File[] getBuiltTemplates() {
        return builtTemplates;
    }

    public File[] getBuiltPassthrough() {
        return builtPassthrough;
    }

    public void setNodes(OtterNode[] n) {
        nodes = n;
    }

    public OtterNode[] getNodes() {
        OtterNode[] n = new OtterTemplateNode[0];

        if (nodes == null) {
            n = getProject().getAllInput();
        } else {
            n = nodes;
        }
        return n;
    }

    protected void buildImpl() throws ToolException {
        File[]    out = new File[0];
        ArrayList outList = new ArrayList();

        if (isInputEnabled()) {
            builtTemplates = makeTemplates();
            if (!getProject().isDeployInputFilter()) {
                builtPassthrough = makePassthrough();
            }
            outList.addAll(Arrays.asList(builtTemplates));
            outList.addAll(Arrays.asList(builtPassthrough));
            out = (File[]) outList.toArray(out);
            outList.clear();
            addFilesToProject(out);
        }
    }

    public void setOwner(Component c) {
        owner = c;
    }

    public Component getOwner() {
        return owner;
    }

    //
    private boolean isInputEnabled() {
        boolean enabled = false;

        if (getNodes().length > 0) {
            enabled = getNodes()[0].getProject().isDeployInput();
        }
        return enabled;
    }

    private void addFilesToProject(File[] files) {
        OtterNodeFactory factory = null;

        factory = getProject().getNodeFactory();
        for (int i = 0; i < files.length; i++) {
            if (!isFresh()) {
                break;
            }
            factory.importFile(getProject(), files[i]);
        }

    }

    private File[] makePassthrough() throws ToolException {
        File[]      out = new File[0];
        OtterFileNode[] pass = new OtterFileNode[0];
        PathHandle  outPath = null;
        File        inFile = null;

        pass = PathUtil.getInputPassthrough(getProject());
        for (int i = 0; i < pass.length; i++) {
            if (pass[i].isSelected()) {
                outPath = getPassthroughPath(pass[i]);
                inFile = new File(pass[i].getFilePath());
                try {
                    FileUtil.copy(inFile, outPath.getFile());
                    echo(res.getString("Copying_input_to")
                         + outPath.getPath());
                } catch (ToolException e) {
                    echo(e);
                    e.printStackTrace();
                }
            }
        }
        return out;
    }

    private File[] makeTemplates() throws ToolException {
        File[]     output = new File[0];
        ConfigTool tool = new ConfigTool();

        tool.setOwner(getOwner());
        tool.setEcho(isEcho());
        tool.setEchoWriter(getEchoWriter());
        tool.setTransformPath(true);
        tool.setTemplates(getTemplates());
        tool.setOverwrite(getProject().isDeployOverwrite());
        tool.initReplacements(PathUtil.expandReplacementTable(getProject(),
                getProject().getReplacementTable()));
        output = tool.createOutput();
        return output;
    }

    private Template[] getTemplates() {
        Template[]          templates = new Template[0];
        ArrayList           list = new ArrayList();
        OtterTemplateNode[] templateNodes = new OtterTemplateNode[0];

        templateNodes = PathUtil.getInputTemplates(getProject());
        for (int i = 0; i < templateNodes.length; i++) {
            OtterTemplateNode cursor = null;
            Template          temp = null;
            File outFile  = null;
            String outPath = new String();

            cursor = templateNodes[i];
            if (cursor.isSelected()) {
                temp = new Template(new File(cursor.getFilePath()),
                                    getProject().getDeployInputPath());
                outPath = cursor.getOutputPath();
                outFile = new File(outPath);
                temp.setOutput(outFile);
                list.add(temp);
            }
            if (! isFresh()) {
              break;
            }
        }
        list.trimToSize();
        templates = new Template[list.size()];
        templates = (Template[]) list.toArray(templates);
        list.clear();
        return templates;
    }

    private PathHandle getPassthroughPath(OtterFileNode node) {
        PathHandle   inputPath = null;
        PathHandle   filePath = null;
        PathHandle   rootPath = null;
        PathHandle   passPath = null;
        StringBuffer buf = new StringBuffer();
        String       suffix = new String();

        inputPath =
            PathHandle.createPathHandle(getProject().getDeployInputPath());
        filePath = PathHandle.createPathHandle(node.getFilePath());
        rootPath =
            PathHandle.createPathHandle(getProject().getDeployRootPath());
        buf.append(rootPath.getPath());
        if (inputPath.parentOf(filePath)) {
            suffix =
                filePath.getPath().substring(inputPath.getPath().length());
        } else {
            suffix = filePath.getFile().getName();
        }
        if ((suffix.length() == 0) || (suffix.charAt(0) != '/')) {
            buf.append('/');
        }
        buf.append(suffix);
        passPath = PathHandle.createPathHandle(buf.toString());
        return passPath;
    }

}
