 /*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.importer;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.TemplateTool;
import org.enhydra.tool.common.Template;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.event.ProgressEvent;
import org.enhydra.tool.common.event.ProgressListener;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ProgressBuilder;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.ValidationUtil;
import org.enhydra.kelp.common.deployer.DeployBuilder;
import org.enhydra.kelp.common.map.Mapper;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterDocumentNode;


// Standard imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 * @author Paul Mahar
 */
public class ImportTool extends ProgressBuilder {

    // string not to be resourced
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res"); // nores
    private final static String KELP_PROPERTIES = "kelp.properties";    // nores
    private final static String PACKAGE_KEY = "package ";    // nores
    private final String[]      IMAGE_TYPES = {
        "gif", "jpg"
    };                                                       // nores

    //
    private ExtensionFilter     optionFilter = null;
    private ImportPaths         paths = null;
    private File[]              inFiles = new File[0];

    /**
     * Method declaration
     *
     *
     * @param javaSource
     *
     * @return
     */
    protected static String readPackage(File javaSource) {
        BufferedReader in = null;
        String         line = null;
        String         pack = null;

        try {
            in = new BufferedReader(new FileReader(javaSource));
            line = in.readLine();
            while (line != null) {
                line = line.trim();
                if (line.startsWith(PACKAGE_KEY)) {
                    pack = line.substring(PACKAGE_KEY.length(),
                                          line.indexOf(';'));
                    pack = pack.trim();
                    break;
                } else {
                    line = in.readLine();
                }
            }
            in.close();
        } catch (java.io.IOException e) {
            line = null;
        }
        if (!ValidationUtil.isJavaPackage(pack)) {
            pack = null;
        }
        return pack;
    }

    /**
     * Constructor declaration
     */
    public ImportTool() {
        super();
        optionFilter = new ExtensionFilter();
        optionFilter.setDirectoryValid(false);
        optionFilter.addExtension(Constants.TYPE_XMLC);
    }

    public ImportPaths getPaths() {
        return paths;
    }

    public void setPaths(ImportPaths p) {
        paths = p;
    }

    /**
     *
     */
    public void addFiles(File[] files) {
        OtterNodeFactory  nodeFactory = null;
        OtterDocumentNode node = null;
        Mapper            mapper = new Mapper(paths.getProject());
        Vector            templateVector = new Vector();
        PathHandle        inputPath = null;
        String[]          docTypes = new String[0];

        docTypes = ToolBoxInfo.getSupportedDocTypes();
        inputPath =
            PathHandle.createPathHandle(paths.getProject().getDeployInputPath());
        nodeFactory = paths.getProject().getNodeFactory();
        for (int i = 0; i < files.length; i++) {
            OtterNode  folder = null;
            PathHandle path = null;

            path = PathHandle.createPathHandle(files[i]);
            if (isIgnored(path.getPath())) {

                // ignore it
            } else {
                folder = getFolder(nodeFactory, inputPath, files[i]);
                if (path.hasExtension(Constants.TYPE_JAVA)) {
                    nodeFactory.createJavaFileNode(folder, path.getPath());

                    // Images
                } else if (path.hasExtension(IMAGE_TYPES)) {
                    nodeFactory.createImageFileNode(folder, path.getPath());

                    // Document resources
                } else if (path.hasExtension(docTypes)) {
                    node = nodeFactory.createDocumentNode(folder,
                                                          path.getPath());
                    setupDocumentNode(mapper, node);

                    // Templates
                } else if (path.hasExtension(Constants.TYPE_IN)
                           && inputPath.parentOf(path)) {
                    OtterTemplateNode template = null;

                    template = nodeFactory.createTemplateNode(folder,
                                                              path.getPath());
                    template.setSelected(true);

                    // Other files xmlc, xml, makefile, rul, conf, etc.
                } else {
                    nodeFactory.createTextFileNode(folder, path.getPath());
                }
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param project
     */
    public void importFilesFromRoot() {
        importFiles(readFilesFromRoot());
    }

    /**
     * Method declaration
     *
     *
     * @param project
     * @param files
     */
    public void importFiles(File[] files) {
        setFresh(true);
        inFiles = files;
        if (getProgressListeners().length == 0) {
            privateImport();
        } else {
            ImportRunner runner = new ImportRunner();
            runner.start();
        }
    }

    private void privateImport() {
        File[] files = inFiles;

        inFiles = new File[0];
        if (isFresh()) {
            refreshProgress(10, res.getString("Preparing_import"));
            preAdd();
        }
        if (isFresh()) {
            refreshProgress(15, res.getString("Searching_templates"));
            files = processConfInSource(files);
        }
        if (isFresh()) {
            refreshProgress(20, res.getString("Importing_files"));
            paths.initPackageMap(true);
        }
        if (isFresh()) {
            addFiles(files);
        }
        if (isFresh()) {
            refreshProgress(75, res.getString("Reading_makefiles"));
            paths.getMakefileReader().updateProject();
        }
        if (isFresh()) {
            refreshProgress(85, res.getString("Processing_input"));
            deployInput();
        }
        if (isFresh()) {
            refreshProgress(95, res.getString("Processing_templates"));
        }
    }

    // Handle E3 newapp generated .conf templates that should
    // be .conf.in
    private File[] processConfInSource(File[] files) {
        TemplateTool   tool = null;
        Template[]     temps = new Template[1];
        File           dest = null;
        PathHandle     filePath = null;
        PathHandle     searchPath = null;

        searchPath = PathHandle.createPathHandle(paths.getSourcePath()
                                                 + File.separator
                                                 + paths.getPackagePath());
        for (int i = 0; i < files.length; i++) {
            if (files[i] == null) {

                // skip
            } else {
                filePath = PathHandle.createPathHandle(files[i]);
                if (filePath.hasExtension(Constants.TYPE_CONF)
                        && searchPath.parentOf(filePath)) {
                    StringBuffer buf = new StringBuffer();

                    buf.append(filePath);
                    buf.append('.');
                    buf.append(Constants.TYPE_IN);
                    dest = new File(buf.toString());
                    if (!dest.exists()) {
                        try {
                            tool = new TemplateTool();
                            temps[0] = new Template(files[i], searchPath.getPath());
                            temps[0].setOutput(dest);
                            tool.setTemplates(temps);
                            files[i] = tool.createOutput()[0];
                        } catch (ToolException e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        }
        return files;
    }

    /**
     *
     */
    private void preAdd() {
        setupProjectSourcePath();
        paths.initProject();
        addKelpProperties();
    }

    /**
     *
     */
    private void deployInput() {
        DeployBuilder  builder = null;
        OtterTemplateNode[] nodes = new OtterTemplateNode[0];

        nodes = PathUtil.getInputTemplates(paths.getProject());
        builder = new DeployBuilder();
        builder.setProject(paths.getProject());
        builder.setInputNodes(nodes);
        builder.setEcho(false);
        builder.buildInput();
        builder.buildContent();

        // Handler for Enhydra 3 style startup
        boolean servlet = false;
        String  multi = new String();
        String  path = new String();

        for (int i = 0; i < nodes.length; i++) {
            path = nodes[i].getOutputPath();
            if (path.endsWith(Constants.FILE_SERVLET_CONF)) {
                servlet = true;
            } else if (path.endsWith(Constants.FILE_MULTISERVER_CONF)) {
                multi = path;
            }
        }
        if ((!servlet) && multi.length() > 0) {
            paths.getProject().setDeployBootstrapPath(multi);
            if (paths.getProject().isDeployStartupRun()) {
                paths.getProject().configureRunClass();
            }
        }
    }

    private boolean isIgnored(String in) {
        boolean ignore = false;
        String  path = in.toLowerCase().trim();

        // conditional strings not to be translated
        if (path.endsWith(".bak")) {                             // nores
            ignore = true;
        } else if (path.endsWith(".class")) {                    // nores
            ignore = true;
        } else if (path.endsWith("~")) {                         // nores
            ignore = true;
        } else if (path.endsWith(".jpr")) {                      // nores
            ignore = true;
        } else if (path.endsWith(".jpx")) {                      // nores
            ignore = true;
        } else if (path.endsWith(".jrl")) {                      // nores
            ignore = true;
        } else if (path.endsWith(".local")) {                    // nores
            ignore = true;
        } else if (path.endsWith("tbl")) {                       // nores
            ignore = true;
        } else if (path.endsWith("tmp")) {                       // nores
            ignore = true;
        } else if (path.endsWith(ImportTool.KELP_PROPERTIES.toLowerCase())) {
            ignore = true;
        } else if (path.indexOf("/cvs/") > -1) {  // nores
            ignore = true;
        } else if (path.indexOf("/classes/") > -1) {             // nores
            ignore = true;
        }
        return ignore;
    }

    /**
     *
     */
    private void addKelpProperties() {
        StringBuffer    filePath = new StringBuffer();
        Properties      properties = new Properties();
        File            file;
        FileInputStream inStream;
        Enumeration     names;
        String          name;
        String          value;

        filePath.append(paths.getProject().getRootPath());
        filePath.append(File.separator);
        filePath.append(KELP_PROPERTIES);
        file = new File(filePath.toString());
        if (file.exists()) {
            try {
                inStream = new FileInputStream(file);
                properties.load(inStream);
                names = properties.propertyNames();
                while (names.hasMoreElements()) {
                    name = (String) names.nextElement();
                    value = properties.getProperty(name);
                    paths.getProject().setProperty(name, value);
                }
                inStream.close();
                properties.clear();
                file.delete();
            } catch (Exception e) {

                // non-fatal, report and continue
                e.printStackTrace();
            }
        }
    }

    private String lookupSymbolicName(File sourceFile) {
        MakefileData[] syms = new MakefileData[0];
        String         symName = new String();

        syms = paths.getMakefileReader().getSymbolicMakefiles();
        for (int i = 0; i < syms.length; i++) {
            if (syms[i].containSourceDoc(sourceFile.getAbsolutePath())) {
                File makeParent = new File(syms[i].getMakeParentPath());

                symName = '.' + makeParent.getName();
                break;
            }
        }
        return symName;
    }

    /**
     *
     */
    private OtterNode getFolder(OtterNodeFactory nodeFactory,
                                PathHandle inputPath,
                                File sourceFile) {

        // Source path + app package + folder + java file name =
        // java file absolute path.
        OtterNode  folderNode = paths.getProject();
        String     symName = new String();
        String     folderName = new String();
        PathHandle projectRoot = null;
        PathHandle sourceRoot = null;
        PathHandle packagePath = null;
        PathHandle outputPath = null;

        folderName = PathHandle.createPathString(sourceFile.getParent());
        projectRoot = PathHandle.createPathHandle(paths.getProject().getRootPath());
        sourceRoot = PathHandle.createPathHandle(paths.getSourcePath());
        outputPath = PathHandle.createPathHandle(paths.getDeployRootPath());
        packagePath = PathHandle.createPathHandle(sourceRoot.getPath()
                                                  + File.separator
                                                  + paths.getPackagePath());

        // check if source is in symbolic makefiles
        symName = lookupSymbolicName(sourceFile);

        //
        if (sourceRoot.parentOf(folderName)) {
            if (packagePath.parentOf(folderName)) {
                folderName =
                    folderName.substring(packagePath.getPath().length() + 1);
                folderName = folderName.replace('/', '.');
            } else {
                folderName =
                    folderName.substring(sourceRoot.getPath().length() + 1);
            }
            folderNode = nodeFactory.createFolderNode(paths.getProject(),
                                                      folderName + symName);
        } else if (projectRoot.parentOf(folderName)) {
            folderName = folderName.substring(projectRoot.getPath().length()
                                              + 1);
            folderNode = nodeFactory.createFolderNode(paths.getProject(),
                                                      folderName + symName);
        } else if (inputPath.parentOf(sourceFile)) {
           folderName = Constants.DIR_INPUT;
           folderNode = nodeFactory.createFolderNode(paths.getProject(),
                                                     folderName + symName);
        } else if (outputPath.parentOf(sourceFile)) {
           folderName = Constants.DIR_OUTPUT;
           folderNode = nodeFactory.createFolderNode(paths.getProject(),
                                                     folderName + symName);

        }
        return folderNode;
    }

    /**
     *
     */
    private void setupDocumentNode(Mapper mapper, OtterDocumentNode node) {
        String     sourcePath = node.getFilePath();
        File       sourceFile = null;
        PathHandle parentPath = null;

        parentPath = PathHandle.createPathHandle(paths.getSourcePath()
                                                 + File.separator
                                                 + paths.getPackagePath());
        if ((sourcePath != null) && (parentPath.parentOf(sourcePath))) {
            sourceFile = new File(sourcePath);
            File   file = null;
            File[] list = new File[0];
            String mappedPath = null;
            String sourceName = null;
            String javaName = null;

            // if .xmlc options file found in mapped path,
            // set xmlc file property
            mappedPath = mapper.getMappedSourcePath(node);
            file = new File(mappedPath);
            file = file.getParentFile();
            list = file.listFiles(optionFilter);
            if ((list != null) && (list.length >= 1)) {
                node.setXMLCOptionFilePath(list[0].getAbsolutePath());
            }
            list = file.listFiles(paths.getJavaFilter());
            if (list != null) {
                for (int i = 0; i < list.length; i++) {
                    sourceName = sourceFile.getName().toLowerCase();
                    sourceName =
                        sourceName.substring(0, sourceName.lastIndexOf('.'));
                    javaName = list[i].getName().toLowerCase();
                    if (javaName.startsWith(sourceName)) {
                        node.setSelected(true);
                        break;
                    }
                }
            }
            if (!node.isSelected()) {
                node.setStatic(true);
            }

            // if mapped package directory contains
            // an xmlc file than set it as the
            // String optionFilename = new String();
            // node.setXMLCOptionFilename(optionFilename);
        }
    }

    /**
     *
     */
    private void setupProjectSourcePath() {
        String[] path = paths.getProject().getSourcePathArray();
        String[] newPath;
        File     current;
        String[] javaList;
        boolean  javaFound = false;

        for (int i = 0; i < path.length; i++) {
            current = new File(path[i]);
            javaList = current.list(new FilenameFilter() {

                /**
                 * Method declaration
                 *
                 * @param dir
                 * @param name
                 *
                 * @return
                 */
                public boolean accept(File dir, String name) {
                    return name.trim().toLowerCase().endsWith(Constants.TYPE_JAVA);
                }

            });
            if ((javaList != null) && (javaList.length >= 1)) {
                javaFound = true;
            }
        }
        if (paths.getSourcePath().length() > 0) {
            if (javaFound) {
                newPath = new String[path.length + 1];
                newPath[0] = paths.getSourcePath();
                for (int i = 1; i < (path.length + 1); i++) {
                    newPath[i] = path[i - 1];
                }
            } else {
                newPath = new String[1];
                newPath[0] = paths.getSourcePath();
            }
            paths.getProject().setSourcePathArray(newPath);
        }
    }

    /**
     *
     */
    private File[] readFilesFromRoot() {
        File[] files = new File[0];

        if (paths.getRootPath() != null) {
            File root = new File(paths.getRootPath());

            files = readFiles(root);
        }
        return files;
    }

    /**
     *
     */
    private File[] readFiles(File dir) {
        File[]     files = new File[0];
        Vector     fileVector = new Vector();
        PathHandle classes = null;
        PathHandle output = null;

        classes = PathHandle.createPathHandle(paths.getClassOutputPath());
        output = PathHandle.createPathHandle(paths.getDeployRootPath());
        if (dir.isDirectory()) {
            File[] list = dir.listFiles();

            for (int i = 0; i < list.length; i++) {
                if (list[i].isDirectory()) {
                    PathHandle cursor = null;

                    cursor = PathHandle.createPathHandle(list[i]);
                    if (classes.parentOf(cursor) || output.parentOf(cursor)) {

                        // skip over classes and output folders.
                    } else {
                        File[] subList = readFiles(list[i]);

                        for (int j = 0; j < subList.length; j++) {
                            fileVector.addElement(subList[j]);
                        }
                    }
                } else if (list[i].isFile()) {
                    fileVector.addElement(list[i]);
                }
            }
        }
        if (fileVector.size() > 0) {
            files = (File[]) fileVector.toArray(files);
        }
        return files;
    }

    private void buildImpl() {
        setFresh(true);
        refreshProgress(5, res.getString("Starting_import"));
        privateImport();
        if (isFresh()) {
            refreshProgress(100, res.getString("Complete"));
        }
    }


    private class ImportRunner extends Thread {
       protected ImportRunner() {
            setPriority((int) ((Thread.MIN_PRIORITY + Thread.NORM_PRIORITY) / 2));
        }

       public void run() {
            buildImpl();
        }

    }


}
