/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.swing;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;

// Standard imports
import javax.swing.*;
import javax.swing.text.BadLocationException;
import java.awt.*;
import java.awt.event.*;
import java.beans.Beans;
import java.io.File;
import java.io.FileWriter;
import java.lang.ref.WeakReference;
import java.util.ResourceBundle;

//
public class OutputPanel extends JPanel implements WriteListener {

    //
    public static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private JScrollPane          scrollOutput;
    private JTextArea            textAreaOutput;
    private BorderLayout         layoutMain;
    private WeakReference        projectRef;
    private int                  waitCount = 0;

    /**
     * Constructor declaration
     *
     */
    public OutputPanel() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void clearAll() {
        clearOutput();
        if (projectRef != null) {
            projectRef.clear();
            projectRef = null;
        }
        removeAll();
    }

    /**
     * Method declaration
     *
     *
     * @param event
     */
    public void onWrite(WriteEvent event) {
        String out = event.getString();

        if (event.getType() == WriteEvent.CLEAR) {
            clearOutput();
        } else {
            if (File.separatorChar == '\\') {

                // string not to be resourced
                if (out.indexOf(":\\") > 0) {
                    out = out.replace('\\', '/');
                }
            }
            addOutput(out);
        }
    }

    public void clearOutput() {
        OutputClear clearRun = null;

        clearRun = new OutputClear(textAreaOutput, scrollOutput);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                clearRun.run();
            } else {
                SwingUtilities.invokeAndWait(clearRun);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void scrollToBottom() {
        OutputScroll runSwing = null;

        runSwing = new OutputScroll(scrollOutput);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                runSwing.run();
            } else {
                SwingUtilities.invokeAndWait(runSwing);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {

            //
        }
    }

    public OtterProject getProject() {
        OtterProject project = null;

        if (projectRef != null) {
            project = (OtterProject) projectRef.get();
        }
        return project;
    }

    public void setProject(OtterProject project) {
        projectRef = new WeakReference(project);
    }

    public int getRows() {
       return textAreaOutput.getRows();
    }

    public void setRows(int i) {
       textAreaOutput.setRows(i);
    }

    //
    // PRIVATE
    //
    private void addOutput(final String output) {
        OutputAdd adder = null;

        adder = new OutputAdd(textAreaOutput, scrollOutput, output);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                adder.run();
            } else {
                waitCount++;
                if (waitCount == 10) {
                    waitCount = 0;
                    SwingUtilities.invokeAndWait(adder);
                } else {
                    SwingUtilities.invokeLater(adder);
                }
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
        projectLog(output);
    }

    private void projectLog(final String output) {
        PathHandle log = null;

        if (getProject() != null) {
            log =
                PathHandle.createPathHandle(getProject().getOutputFilename());
            if (log.hasExtension("txt") && log.getFile().canWrite()) {
                try {
                    FileWriter writer = null;

                    writer = new FileWriter(log.getPath(), true);
                    writer.write(output);    // line + new Character((char)13) + new Character((char)10));
                    writer.close();
                } catch (java.io.IOException e) {
                    e.printStackTrace(System.err);
                }
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        textAreaOutput =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());
        textAreaOutput.setEditable(false);
        textAreaOutput.setRows(15);
        scrollOutput = new JScrollPane(textAreaOutput);
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        this.setLayout(layoutMain);
        this.add(scrollOutput, BorderLayout.CENTER);
    }

    //
    private class OutputAdd implements Runnable {
        private JTextArea   textArea = null;
        private JScrollPane scroll = null;
        private String      out = null;

        protected OutputAdd(JTextArea ta, JScrollPane sc, String o) {
            textArea = ta;
            scroll = sc;
            out = o;
        }

        synchronized public void run() {
            int start = 0;
            int end = 0;

            if (textArea.getLineCount() > 2000) {
                try {
                    start = textArea.getLineStartOffset(0);
                    end = textArea.getLineEndOffset(0);
                } catch (BadLocationException e) {
                    start = 0;
                    end = 0;
                }
                textArea.replaceRange(null, start, end);
            }
            textArea.append(out);
            scroll.getVerticalScrollBar().setValue(scroll.getVerticalScrollBar().getMaximum());
            textArea.repaint();
            repaint();
        }

    }

    //
    private class OutputClear implements Runnable {
        private JTextArea   textArea = null;
        private JScrollPane scroll = null;

        protected OutputClear(JTextArea ta, JScrollPane sc) {
            textArea = ta;
            scroll = sc;
        }

        synchronized public void run() {
            textArea.setText(new String());
            scroll.getVerticalScrollBar().setValue(scroll.getVerticalScrollBar().getMaximum());
            textArea.repaint();
            repaint();
        }

    }

    //
    private class OutputScroll implements Runnable {
        private JScrollPane scroll = null;

        protected OutputScroll(JScrollPane sc) {
            scroll = sc;
        }

        synchronized public void run() {
            scroll.getVerticalScrollBar().setValue(scroll.getVerticalScrollBar().getMaximum());
        }

    }
}
