/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.xmlc;

// ToolBox
import org.enhydra.tool.common.InnerPanel;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.event.FirstFocusEvent;
import org.enhydra.tool.common.event.FirstFocusListener;

// AddinCore
import org.enhydra.kelp.common.AbstractTool;
import org.enhydra.kelp.common.event.SelectionPanelEvent;
import org.enhydra.kelp.common.event.SelectionPanelListener;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.swing.AddinInnerPanel;
import org.enhydra.kelp.common.properties.XMLCProjectPropertyPanel;

// Standard imports
import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.lang.ref.WeakReference;
import java.util.ResourceBundle;

//
public class CoreXMLCTool extends AbstractTool
    implements SelectionPanelListener, FirstFocusListener {
    static ResourceBundle            addinRes =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    //
    private XMLCNodeBuilder          nodeBuilder = null;
    private XMLCProjectPropertyPanel propertyPanel = null;
    private XMLCInnerPanel           innerPanel = null;
    private XMLCButtonPanel          buttonPanel = null;
    private WeakReference            compileRef = null;

    public static void main(String[] args) {
        SwingUtil.setLookAndFeelToSystem();
        CoreXMLCTool tool = null;

        tool = new CoreXMLCTool();
        tool.showDialog(new Frame());
        System.exit(0);
    }

    public static final String getDefaultTitle() {
        return "XML Compiler";
    }

    public CoreXMLCTool() {
        super();
        innerPanel = new XMLCInnerPanel();
        buttonPanel = new XMLCButtonPanel();
        innerPanel.getSelectionPanel().addSelectionPanelListener(this);
        innerPanel.setPropertyPanel(getPropertyPanel());
        innerPanel.setSelectText(getSelectText());
        innerPanel.addFirstFocusListener(this);
    }

    // implements SelectionPanelListener
    public void onSelection(SelectionPanelEvent event) {
        boolean enable = event.getSelected().size() > 0;

        refreshCompile(enable);
    }

    // implements AbstractTool
    public String getTitle() {
        return CoreXMLCTool.getDefaultTitle();
    }

    // implements AbstractTool
    public InnerPanel getInnerPanel() {
        return innerPanel;
    }

    // implements AbstractTool
    public ButtonPanel getButtonPanel() {
        return buttonPanel;
    }

    public ActionListener createButtonListener() {
        return (new XMLCButtonListener(this));
    }

    // overrides AbstractTool
    public void setProject(OtterProject project) {
        super.setProject(project);
        setNodes(project.getAllXMLCNodes());
    }

    // overrides AbstractTool
    public void clearAll() {
        super.clearAll();
       innerPanel.removeFirstFocusListener(this);
        if (compileRef != null) {
            compileRef.clear();
        }
        innerPanel = null;
        buttonPanel = null;
        compileRef = null;
    }

    public void build() {
        innerPanel.selectOutputTab();
        innerPanel.save();
        innerPanel.clearOutput();
        openProgress();
        buildImpl();
    }

    // implements FirstFocusListener
    public void onFirstFocus(FirstFocusEvent event) {
      refreshCompile();
    }

    //
    // PROTECTED
    //
    // implements org.enhydra.kelp.common.tool.AbstractTool

    /**
     * Get source files that can be selected for compilation.
     */
    protected OtterFileNode[] getNodes() {
        return innerPanel.getNodes();
    }

    /**
     * Set the source files that can be selected for compilation
     */
    protected void setNodes(OtterFileNode[] n) {
        innerPanel.setNodes(n);
        refreshCompile();
    }

    protected String[] getSelectText() {
        String[] s = {
            "Documents Available", "Documents Selected"
        };

        return s;
    }

    // implements org.enhydra.kelp.common.tool.AbstractTool
    protected String getBuildText() {
        return addinRes.getString("buttonMake_Text");
    }

    // implements org.enhydra.kelp.common.tool.AbstractTool
    protected String getProgressTitle() {
        return addinRes.getString("XMLC_Progress");
    }

    // implements org.enhydra.kelp.common.tool.AbstractTool
    protected AddinInnerPanel getPropertyPanel() {
        if (propertyPanel == null) {
            propertyPanel = new XMLCProjectPropertyPanel();
        }
        return propertyPanel;
    }

    private void buildImpl() {
        if (nodeBuilder != null) {
            nodeBuilder.setFresh(false);
        }
        nodeBuilder = new XMLCNodeBuilder(innerPanel.getWriteListeners());
        nodeBuilder.setProject(getProject());
        nodeBuilder.setNodes(getProject().getAllXMLCNodes());
        nodeBuilder.addProgressListener(getProgressMeter());
        getProgressMeter().addCancelListener(nodeBuilder);
        nodeBuilder.build();
    }

    private Component getCompile() {
        Component comp = null;

        if (compileRef == null) {
            comp =
                ButtonPanel.findButton(getInnerPanel().getTopLevelAncestor(),
                                       XMLCButtonPanel.COMMAND_COMPILE);
            if (comp != null) {
                compileRef = new WeakReference(comp);
            }
        } else {
            comp = (Component) compileRef.get();
        }
        return comp;
    }

    private void refreshCompile() {
        boolean         enable = false;
        OtterFileNode[] nodes = new OtterFileNode[0];

        nodes = innerPanel.getNodes();
        for (int i = 0; i < nodes.length; i++) {
            if (nodes[i].isSelected()) {
                enable = true;
                break;
            }
        }
        refreshCompile(enable);
    }

    private void refreshCompile(boolean enable) {
        Component compileButton = getCompile();

        if (compileButton != null) {
            compileButton.setEnabled(enable);
        }
    }

}
